'' ******************************************************************************
'' * LCD Object                                                                 *
'' * James Burrows May 2006                                                     *
'' * Version 0.5                                                                *
'' ******************************************************************************
''
'' for reference look at: www.milinst.com - Audio Visual section
'' portions adapted from Parallax's SimpleNumbers library
''
'' this object provides the PUBLIC functions:
''  -> Init  - sets up the address and inits sub-objects such
''  -> Start - init the serial and setup
''  -> Stop  - stop the serial and release the IO pin
''  -> putlinec - put a character to the LCD on a specific line
''  -> putlines - put a string to the LCD on a specific line 
''  -> clearLCD - Clear the lcd
''  -> positionLCD - set the cursor position. Valid: lines (1-4), chars (0-19)
''  -> output - output byte
''  -> outputstring - output a string to current cursor loc 
''  -> outputnumber - output a number to the current cursor loc  
''
'' this object provides the PRIVATE functions:
''  -> none
''
'' this object uses the following sub OBJECTS:
''  -> SimpleSerial (tx only!)
''  -> SimpleNumbers
''

CON
  LCDCommand    = $FE
  LCDClear      = $1


VAR
  word  LineLength
  word  LineDebugCount
  long  div,zpad
  long  running
  word  LCD_Pin, LCD_Baud

obj
  SimpleNumbers  : "Simple_Numbers"
  serial         : "fullduplexserial" 

  
PUB  Init(_LCD_Pin, _LCD_Baud, _lineLength): okay
  LineLength := _lineLength
  LCD_Pin := _LCD_Pin
  LCD_Baud := _LCD_Baud
  running := false
  start


PUB  Start
  if running == false
    running := true
    LineDebugCount := 0
     serial.start(-1,LCD_Pin,3,LCD_Baud)


PUB  Stop
  if running == true
    serial.stop  
    running := false

    
PUB  Changeline(lineref,str_address)
  clearline(lineref)
  outputstring(str_address)

  
PUB  ClearLCD
  if running == true
    serial.tx(LCDCommand)
    serial.tx(LCDClear)

    
PUB  ClearLine(lineref)
  if running == true
    positionLCD(lineref,0)
    repeat lineLength
      serial.tx(" ")
    positionLCD(lineref,0)


PUB DebugChar(lineRef, strPointer)
  if running == true
    if LineDebugCount == LineLength
      LineDebugCount :=0
     
    positionLCD (lineRef,LineDebugCount)
    LineDebugCount++
    output (strPointer)    

PUB BigChar(lcdNumber, CharRef)

   case lcdnumber
      0:   outputstringX(1,CharRef,string("  -- "))
           outputstringX(2,CharRef,string(" |  |"))
           outputstringX(3,CharRef,string(" |  |"))
           outputstringX(4,CharRef,string("  -- "))
           
      1:   outputstringX(1,CharRef,string("  -| "))
           outputstringX(2,CharRef,string("   | "))
           outputstringX(3,CharRef,string("   | "))
           outputstringX(4,CharRef,string(" ----"))
           
      2:   outputstringX(1,CharRef,string(" --- "))
           outputstringX(2,CharRef,string("  __|"))
           outputstringX(3,CharRef,string(" |   "))
           outputstringX(4,CharRef,string(" ----"))

      3:   outputstringX(1,CharRef,string(" --- "))
           outputstringX(2,CharRef,string("  __|"))
           outputstringX(3,CharRef,string("    |"))
           outputstringX(4,CharRef,string(" --- "))

      4:   outputstringX(1,CharRef,string(" |   "))
           outputstringX(2,CharRef,string(" | | "))
           outputstringX(3,CharRef,string(" ----"))
           outputstringX(4,CharRef,string("   | "))

      5:   outputstringX(1,CharRef,string("  -- "))
           outputstringX(2,CharRef,string(" |__ "))
           outputstringX(3,CharRef,string("    |"))
           outputstringX(4,CharRef,string("  -- "))

      6:   outputstringX(1,CharRef,string("  -- "))
           outputstringX(2,CharRef,string(" |__ "))
           outputstringX(3,CharRef,string(" |  |"))
           outputstringX(4,CharRef,string("  -- "))

      7:   outputstringX(1,CharRef,string(" --- "))
           outputstringX(2,CharRef,string("   / "))
           outputstringX(3,CharRef,string("  /  "))
           outputstringX(4,CharRef,string(" /   "))

      8:   outputstringX(1,CharRef,string("  -- "))
           outputstringX(2,CharRef,string(" |__|"))
           outputstringX(3,CharRef,string(" |  |"))
           outputstringX(4,CharRef,string("  -- "))

      9:   outputstringX(1,CharRef,string("  -- "))
           outputstringX(2,CharRef,string(" |__|"))
           outputstringX(3,CharRef,string("    |"))
           outputstringX(4,CharRef,string("    |"))    
    
PUB  output(value)
  if running == true
    serial.tx(value)

    
PUB  outputc(character)
  if running == true
     case character
       32      : serial.tx(character)
       36      : serial.tx(character)       
       46..57  : serial.tx(character)
       65..90  : serial.tx(character)
       97..122 : serial.tx(character)  

  
PUB  PositionLCD (lineref,charRef)
  ' 128 / 192 / 148 / 212
  if running == true  
    serial.tx(LCDCommand)
    case lineref
      1 : serial.tx(128 + charref)
      2 : serial.tx(192 + charref)
      3 : serial.tx(148 + charref)
      4 : serial.tx(212 + charref)
      other : serial.tx(128)
      
      
PUB  OutputString(str_addr) | strLength, counter, character
  ' output a STRING to the current X,Y
  if running == true
    strLength := strSize(str_addr) -1  
    if strLength > LineLength
      strLength := LineLength
    repeat counter from 0 to strLength
       character := byte[str_addr][counter]
       case character
         32      : serial.tx(character)
         36      : serial.tx(character)          
         46..57  : serial.tx(character)
         65..90  : serial.tx(character)
         97..122 : serial.tx(character)   


PUB OutputStringX(lineRef, charRef, strPointer)
   if running == true
      positionLCD(lineref, charRef)
    repeat strsize(strPointer)                            ' for each character in string
      serial.tx(byte[strPointer++])         
         
PUB outputHex(value,digits)
  ' output a HEX number to the current X,Y
  if running == true
    outputstring (simplenumbers.hex(value,digits))        

    
PUB  outputNumber(value)
  ' output a dec number to the current X,Y
  if running == true
  '  if value => 48 and value =< 57 
      outputstring (simplenumbers.dec(value))


PUB outputNumberX(value, digits)
  if running == true
    outputstring (simplenumbers.decx(value, digits))
    
PUB outputBinary(value, digits)
  ' output a binary number to the current X,Y
  if running == true
    outputstring (simplenumbers.bin(value,digits))

    