'' ******************************************************************************
'' * BigMemObject Object                                                        *
'' * James Burrows June 2006                                                    *
'' * Version 1.1                                                                *
'' ******************************************************************************
''
''  This object allows a PC to act as a large block of RAM (bigger than the propeller)
''  can at the current time.
''
''  The communication is via 115_200 serial comm port
''
''  The PC needs to be running the VB6 app "C64RAM" first!
''
''  Any communication breaks (readValue) will return a 0 value.  No error handling here
''  just a time-out to allow the application to continue
''
'' Version 1.0 = release
''         1.1 = changed to readValue & writeValue
''               changed to allow LONG read and writes (vb app changed as well)
''               error handling : if comm failure between VB and Propeller the
''                                readValue routine now times out.
''
'' this object provides the PUBLIC functions:
''  -> Init  - sets up the serial pins. Baud is forced
''  -> WriteValue(RAMaddress, value) - writes to the PC ram
''  -> ReadValue(ramAddress) : bytevalue - returns from the pc ram
''
'' this object provides the PRIVATE functions:
''  -> None
''
'' this object uses the following sub OBJECTS:
''  -> simpleNumbers
''  -> fullduplexserial
''

CON
  ' these are the markers used in the data stream
  AddressMarker = 169
  ReadMarker    = 170
  ReadMarker2   = 173
  WriteMarker1  = 171
  WriteMarker2  = 172

  ' this is the number of -1's allowed before terminating the read
  NoByteMaxDelay = 20
  ' this is the rxTime delay (ms) before giving up the wait
  ByteMaxWaitDelay = 100

VAR
  long  running

OBJ
  uart                : "fullduplexserial"
  num                 : "simple_numbers"   

PUB Init(_rxPin, _txPin): result
  ' init the object - rx and tx pins - the baud is forced as the VB is set to 115_200
  uart.start(_rxPin, _txPin, %0000, 115_200)  
  running := true
  return running  


PUB WriteValue(RAMaddress, value)
  ' send an address
  if running == true     
    uart.tx(AddressMarker)
    uart.str(num.dec(RAMaddress))

    ' write marker
    uart.tx(WriteMarker1)
    uart.str(num.dec(value))
    uart.tx(WriteMarker2)


PUB ReadValue(ramAddress) : bytevalue | inbyte, noByteCount
  ' read an address from the PC
  if running == true
    uart.tx(AddressMarker)
    uart.str(num.dec(ramAddress))
    uart.tx(ReadMarker)       
     
    ' get the data - must wait for it here
    bytevalue := 0
    noByteCount := 0
    inbyte := uart.rxTime(ByteMaxWaitDelay)
        
    repeat until inbyte == ReadMarker2 or noByteCount > NoByteMaxDelay
      ' add to number if not -1
      if inbyte <> - 1
        bytevalue *= 10
        bytevalue += inbyte - 48
        inbyte := uart.rxTime(ByteMaxWaitDelay)
      else
        noByteCount++
      
    ' return value 
    return bytevalue   